#ifndef SOERGEL_H
#define SOERGEL_H

/*
 *  soergel.h
 *  Mothur
 *
 *  Created by westcott on 12/15/10.
 *  Copyright 2010 Schloss Lab. All rights reserved.
 *
 */


#include "calculator.h"

/***********************************************************************/

class Soergel : public Calculator  {
	
public:
	Soergel() :  Calculator("soergel", 1, false) {};
	EstOutput getValues(SAbundVector*) {return data;};
	EstOutput getValues(vector<SharedRAbundVector*>);
	string getCitation() { return "http://www.mothur.org/wiki/Soergel"; }
private:
	
};

/***********************************************************************/

#endif



